{-# LANGUAGE TemplateHaskell #-}

module Feature.Generation.Passwords
  ( Password
  , Random
  , suggestedScheme
  , xkcd
  , oldschool
  , gibberish
  ) where

import           Core.Types
import           Feature.Generation.Shared (titleCase, rCharSym, rIndex,
                                            validChars, validNumbers,
                                            validSymbols)

import           Data.FileEmbed
import           Data.List                 (singleton)

{- Suggests one of Purr's generation schemes based on the
   desired number of characters in the password. -}
suggestedScheme :: Int -> Random Password
suggestedScheme i
  | i > 17    = xkcd
  | i > 12    = oldschool
  | otherwise = gibberish i

-- XKCD-style random password generator consisting of four titlecase words.
xkcd :: Random Password
xkcd = do
  wOne   <- rTitle
  wTwo   <- rTitle
  wThree <- rTitle
  wFour  <- rTitle
  return  $ Password' (wOne <> wTwo <> wThree <> wFour)

-- Two random title case words, four random numbers, one random symbol.
oldschool :: Random Password
oldschool = do
  wOne <- rTitle
  wTwo <- rTitle
  nOne <- rNum
  nTwo <- rNum
  nThr <- rNum
  nFou <- rNum
  sOne <- rSym
  return
    $ Password' (wOne <> wTwo
   <> show nOne <> show nTwo <> show nThr <> show nFou
   <> pure sOne)

-- A completely random selection of characters supported by Purr generation
gibberish :: Int -> Random Password
gibberish i = go i (return "")
  where
    go :: Int -> Random String -> Random Password
    go 0 s = Password' <$> s
    go i s = go (i - 1) (s <> (singleton <$> rCharSym))

rNum :: Random Int
rNum = rIndex validNumbers

rWord :: Random String
rWord = rIndex wordList

rSym :: Random Char
rSym = rIndex validSymbols

rTitle :: Random String
rTitle = titleCase <$> rWord

wordList :: [String]
wordList = lines $(embedStringFile "data/assets/wordlist.txt")
