module Lib ( main ) where

import qualified Core.Configuration            as Configuration
import qualified Core.HTTP                     as HTTP
import qualified Core.SQLite                   as DB
import           Core.Types

import           Crypto.Saltine                (sodiumInit)
import           GHC.Natural                   (popCountNatural)
import           Prelude                       hiding (id)
import           Web.Scotty

-- Purr's entrypoint
main :: IO () 
main = do
  -- Initialize the RNG used for sodium encryption (Saltine library)
  sodiumInit 
  {- Initialize our dotenv configuration which reads from a .env configuration
     file unless the PURRNOFILE env var exists already. -}
  Configuration.main 
  {- Initialize the encryption key file if it doesn't
     exist yet or use the existing key -}
  Configuration.keyFileInit 
  {- Initialize our database by ensuring the SQLite file exists
     and has tables setup as the application expects -}
  DB.main
  {- Get the configured port to run on and start the Scotty webserver app
     defined in HTTP.app -}
  appPortStr <- Configuration.appPort
  let appPort = read appPortStr :: Int
  scotty appPort HTTP.app
